/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

/* ====================================================================
 * Copyright (c) 2002, The Regents of the University of California.
 *
 * The Regents of the University of California MAKE NO REPRESENTATIONS OR
 * WARRANTIES ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The Regents of the University of California SHALL
 * NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF
 * USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * ====================================================================
 */

#ifndef __dav_repos_H__
#define __dav_repos_H__

#include <mod_dav.h>
#include "dbms_mysql.h"

/* REMOVE DEBUG INFO */
#ifndef DEBUG
#undef DBG0
#undef DBG1
#undef DBG2
#undef DBG3
#define DBG0(A)
#define DBG1(A,B)
#define DBG2(A,B,C)
#define DBG3(A,B,C,D)
#endif

#define TRACE() DBG1("- TRACE : %s",  __func__ )

#ifndef VERSION
#define VERSION "static"
#endif

#ifdef __CYGWIN__
#define atoll(x)	strtoll(x, 0, 10)
#define atoull(x)	strtoull(x, 0, 10)
#endif


typedef struct
{
#ifdef USE_DIR
    const char *db_host;
    const char *db_name;
    const char *db_id;
    const char *db_pass;
    const char *tmp_dir;
    const char *file_dir;
    long max_file_size;
    int exact_search;

    apr_table_t *d_params;	/* per-directory DAV config parameters */

    dav_repos_dbms db;
#endif
}
dav_repos_dir_conf;


typedef struct
{
    const char *db_host;
    const char *db_name;
    const char *db_id;
    const char *db_pass;
    const char *tmp_dir;
    const char *file_dir;
    long max_file_size;
    int exact_search;

    apr_table_t *d_params;	/* per-directory DAV config parameters */

    dav_repos_dbms db;
}
dav_repos_server_conf;


/* dav_repos_server_conf is in repos.h */
typedef dav_repos_server_conf dav_repos_db;


/* our hooks structures; these are gathered into a dav_provider */
extern const dav_hooks_repository dav_repos_hooks_repos;
extern const dav_hooks_propdb     dav_repos_hooks_propdb;
extern const dav_hooks_search     dav_repos_hooks_search;
extern const dav_hooks_liveprop   dav_repos_hooks_liveprop;
extern const dav_hooks_vsn        dav_repos_hooks_vsn;
extern const dav_hooks_locks      dav_repos_hooks_locks;

extern const dav_liveprop_group dav_repos_liveprop_group;

/* handle the SEARCH method */
int dav_repos_method_search(request_rec * r);

void dav_repos_insert_all_liveprops(request_rec * r,
				    const dav_resource * resource,
				    dav_prop_insert what,
				    apr_text_header * phdr);
int dav_repos_find_liveprop(const dav_resource * resource, const char *ns_uri,
			    const char *name,
			    const dav_hooks_liveprop ** hooks);


/* some prototype */
const char *dav_repos_getetag(const dav_resource * resource);

#endif








