/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

/* ====================================================================
 * Copyright (c) 2002, The Regents of the University of California.
 *
 * The Regents of the University of California MAKE NO REPRESENTATIONS OR
 * WARRANTIES ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The Regents of the University of California SHALL
 * NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF
 * USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 * ====================================================================
 */
/*
 * mod_dav_svn.c: an Apache mod_dav dbms-module
 *               
 */

#include <httpd.h>
#include <http_config.h>
#include <http_protocol.h>
#include <http_log.h>
#include <http_core.h>		/* for ap_construct_url */
#include <mod_dav.h>

#include <apr_strings.h>
#include <apr_hash.h>

#include "dav_repos.h"
#include "dbms.h"
#include "mod_dav_repos_version.h"

#define INHERIT_VALUE(parent, child, field) \
                ((child)->field ? (child)->field : (parent)->field)

#define DBMS_PRIVIDER_NAME "repos"

/* Note: the "dav_repos" prefix is mandatory */
extern module AP_MODULE_DECLARE_DATA dav_repos_module;

static void *dav_repos_create_dir_config(apr_pool_t * p, char *dir)
{
    /* NOTE: dir==NULL creates the default per-dir config */
    dav_repos_dir_conf *conf;

    conf = (dav_repos_dir_conf *) apr_pcalloc(p, sizeof(*conf));

    return conf;
}

static void *dav_repos_merge_dir_config(apr_pool_t * p, void *base,
					void *overrides)
{
    /** FIXME : for future use
    dav_repos_dir_conf *parent = base;
    dav_repos_dir_conf *child = overrides;
    */
    dav_repos_dir_conf *newconf;

    newconf = apr_pcalloc(p, sizeof(*newconf));
    return newconf;
}

static void *dav_repos_create_server_config(apr_pool_t * p, server_rec * s)
{
    dav_repos_server_conf *newconf;
    newconf = (dav_repos_server_conf *) apr_pcalloc(p, sizeof(*newconf));
    return newconf;
}

static void *dav_repos_merge_server_config(apr_pool_t * p, void *base,
					   void *overrides)
{
    dav_repos_server_conf *parent = base;
    dav_repos_server_conf *child = overrides;

    dav_repos_server_conf *newconf;
    newconf = (dav_repos_server_conf *) apr_pcalloc(p, sizeof(*newconf));

    newconf->db_host = INHERIT_VALUE(parent, child, db_host);
    newconf->db_name = INHERIT_VALUE(parent, child, db_name);
    newconf->db_id = INHERIT_VALUE(parent, child, db_id);
    newconf->db_pass = INHERIT_VALUE(parent, child, db_pass);
    newconf->tmp_dir = INHERIT_VALUE(parent, child, tmp_dir);
    newconf->file_dir = INHERIT_VALUE(parent, child, file_dir);
    newconf->exact_search = INHERIT_VALUE(parent, child, exact_search);
    newconf->max_file_size = INHERIT_VALUE(parent, child, max_file_size);

    return newconf;
}


/*
 * Command handler for the DAV directive, which is TAKE1.
 */
static const char *dav_repos_db_host_cmd(cmd_parms * cmd, void *config,
					 const char *arg1)
{
    dav_repos_server_conf *conf =
	ap_get_module_config(cmd->server->module_config,
			     &dav_repos_module);
    conf->db_host = apr_pstrdup(cmd->pool, arg1);
    return NULL;
}


static const char *dav_repos_db_name_cmd(cmd_parms * cmd, void *config,
					 const char *arg1)
{
    dav_repos_server_conf *conf =
	ap_get_module_config(cmd->server->module_config,
			     &dav_repos_module);

    conf->db_name = apr_pstrdup(cmd->pool, arg1);
    return NULL;
}

static const char *dav_repos_db_id_cmd(cmd_parms * cmd, void *config,
				       const char *arg1)
{
    dav_repos_server_conf *conf =
	ap_get_module_config(cmd->server->module_config,
			     &dav_repos_module);

    conf->db_id = apr_pstrdup(cmd->pool, arg1);
    return NULL;
}

static const char *dav_repos_db_pass_cmd(cmd_parms * cmd, void *config,
					 const char *arg1)
{
    dav_repos_server_conf *conf =
	ap_get_module_config(cmd->server->module_config,
			     &dav_repos_module);

    conf->db_pass = apr_pstrdup(cmd->pool, arg1);
    return NULL;
}

static const char *dav_repos_tmp_dir_cmd(cmd_parms * cmd, void *config,
					 const char *arg1)
{
    dav_repos_server_conf *conf =
	ap_get_module_config(cmd->server->module_config,
			     &dav_repos_module);

    conf->tmp_dir = apr_pstrdup(cmd->pool, arg1);
    return NULL;

}

static const char *dav_repos_file_dir_cmd(cmd_parms * cmd, void *config,
					 const char *arg1)
{
    dav_repos_server_conf *conf =
	ap_get_module_config(cmd->server->module_config,
			     &dav_repos_module);

    conf->file_dir = apr_pstrdup(cmd->pool, arg1);
    return NULL;

}

// Maximun File Size for storing in DB. This is at testing-stage. 
static const char *dav_repos_max_f_size_cmd(cmd_parms * cmd, void *config,
					 const char *arg1)
{
    dav_repos_server_conf *conf =
	ap_get_module_config(cmd->server->module_config,
			     &dav_repos_module);

    conf->max_file_size = atol(arg1);
    return NULL;

}

static const char *dav_repos_exact_search_cmd(cmd_parms *cmd, void *config, const int arg)
{
    dav_repos_server_conf *conf =
	ap_get_module_config(cmd->server->module_config,
			     &dav_repos_module);

    conf->exact_search = arg;
    return NULL;
}

static const command_rec dav_repos_cmds[] = {
    /* per directory/location */
    /* how can I make it mandatory */
    /* RSRC_CONF : For server */
    /* ACCESS_CONF : For dir */
    AP_INIT_TAKE1("DavDBMSHost", dav_repos_db_host_cmd, NULL, RSRC_CONF,
		  "specify the MYSQL_DB_HOST for a directory or location"),

    AP_INIT_TAKE1("DavDBMSDbName", dav_repos_db_name_cmd, NULL, RSRC_CONF,
		  "specify the MYSQL_DB_NAME for a directory or location"),

    AP_INIT_TAKE1("DavDBMSId", dav_repos_db_id_cmd, NULL, RSRC_CONF,
		  "specify the MYSQL_DB_ID for a directory or location"),

    AP_INIT_TAKE1("DavDBMSPass", dav_repos_db_pass_cmd, NULL, RSRC_CONF,
		  "specify the MYSQL_DB_PASS for a directory or location"),

    AP_INIT_TAKE1("DavDBMSTmpDir", dav_repos_tmp_dir_cmd, NULL, RSRC_CONF,
		  "specify the MYSQL_TMP_DIR for a directory or location"),

    AP_INIT_TAKE1("DavDBMSFileDir", dav_repos_file_dir_cmd, NULL, RSRC_CONF,
		  "specify the directory for permanent external storage"),

    AP_INIT_FLAG("DavDBMSUseExactSearch", dav_repos_exact_search_cmd, NULL, RSRC_CONF,
		 "specify whether to use exact FULLTEXT search, or the relevancy heuristics of MySQL"),

    AP_INIT_TAKE1("DavDMBSMaxFileSize", dav_repos_max_f_size_cmd, NULL, RSRC_CONF,
		  "specify the directory for permanent external storage"),

    {NULL}
};

/** Add DASL/0.1 version */
static int dav_repos_init_handler(apr_pool_t * p, apr_pool_t * plog,
				  apr_pool_t * ptemp, server_rec * s)
{
    DBG0("init_handler");

    ap_add_version_component(p, "Catacomb/" VERSION);
    return OK;
}

/* Provider interface */
static const dav_provider dav_repos_provider = {
    &dav_repos_hooks_repos,	/* repos */
    &dav_repos_hooks_propdb,	/* prop */
    &dav_repos_hooks_locks,	/* locks */
    &dav_repos_hooks_vsn,       /* versioning */
    NULL,			/* binding */
    &dav_repos_hooks_search	/* search */
};

/*
 * Closing database when httpd exit
 */
static apr_status_t dav_repos_child_exit(void *data)
{
    server_rec *s = data;
    dav_repos_db *db =
	ap_get_module_config(s->module_config, &dav_repos_module);

    /** FIXME: Error handling?? */
    dbms_closedb(db);

    /** FIXME : what should I return? */
    return 0;
}

/*
 * Open database when httpd start
 */
static void dav_repos_child_init(apr_pool_t * p, server_rec * s)
{
    dav_repos_db *db =
	ap_get_module_config(s->module_config, &dav_repos_module);

    /*#FIXME
       ** open db
       ** How can I report the Error?? 
     */
    dbms_opendb(db);

    /* register child_exit */
    apr_pool_cleanup_register(p, s, dav_repos_child_exit,
			      dav_repos_child_exit);
}

/*
static int logging_of_user_data(request_rec *r)
{
	//ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, "Markus-Test-Modul" ": uri=%s user=%s filename=%s",
	//					 r->uri,r->user,r->filename);
	return DECLINED;
}
*/

static void register_hooks(apr_pool_t * p)
{
    /* apache hooks */
    ap_hook_post_config(dav_repos_init_handler, NULL, NULL, APR_HOOK_MIDDLE);
    ap_hook_child_init(dav_repos_child_init, NULL, NULL, APR_HOOK_MIDDLE);
    /* test_hook by mlitz */
    //ap_hook_fixups(logging_of_user_data, NULL, NULL, APR_HOOK_MIDDLE);

    /* live property handling */
    dav_hook_find_liveprop(dav_repos_find_liveprop, NULL, NULL,
			   APR_HOOK_MIDDLE);
    dav_hook_insert_all_liveprops(dav_repos_insert_all_liveprops, NULL, NULL,
				  APR_HOOK_MIDDLE);

    /* register the namespace URIs */
    dav_register_liveprop_group(p, &dav_repos_liveprop_group);

    /* register our provider */
    dav_register_provider(p, DBMS_PRIVIDER_NAME, &dav_repos_provider);
   
}


module DAV_DECLARE_DATA dav_repos_module = {
    STANDARD20_MODULE_STUFF,
    dav_repos_create_dir_config,	/* dir config creater */
    dav_repos_merge_dir_config,	/* dir merger --- default is to override */
    dav_repos_create_server_config,	/* server config */
    dav_repos_merge_server_config,	/* merge server config */
    dav_repos_cmds,		/* command table */
    register_hooks,		/* register hooks */
};
